/* Copyright 2010 ESRI
* 
* All rights reserved under the copyright laws of the United States
* and applicable international laws, treaties, and conventions.
* 
* You may freely redistribute and use this sample code, with or
* without modification, provided you include the original copyright
* notice and use restrictions.
* 
* See the use restrictions at http://help.arcgis.com/en/sdk/10.0/usageRestrictions.htm
* 
*/
/*
 * Copyright (c) 2009 ESRI All rights reserved under the copyright laws of the United States and applicable
 * international laws, treaties, and conventions. You may freely redistribute and use this sample code, with or without
 * modification, provided you include the original copyright notice and use restrictions. See use restrictions at
 * /arcgis/java/samples/userestrictions.
 */

package arcgissamples.trackinganalyst.command;

import java.awt.BorderLayout;
import java.io.File;
import java.io.IOException;
import java.net.UnknownHostException;
import java.util.Random;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JProgressBar;

import com.esri.arcgis.carto.*;
import com.esri.arcgis.controls.BaseCommand;
import com.esri.arcgis.controls.HookHelper;
import com.esri.arcgis.controls.IHookHelper;
import com.esri.arcgis.controls.IToolbarControl2;
import com.esri.arcgis.datasourcesfile.ShapefileWorkspaceFactory;
import com.esri.arcgis.display.*;
import com.esri.arcgis.geodatabase.*;
import com.esri.arcgis.interop.AutomationException;
import com.esri.arcgis.system.*;
import com.esri.arcgis.trackinganalyst.*;

import com.esri.arcgis.support.ms.stdole.*;

public class DisplayDataCommand extends BaseCommand
{
	private IHookHelper hookHelper = null;
	private boolean isConnected = false;
	private boolean checked = false;
	private TemporalFeatureLayer temporalLayer = null;
	private InvokeHelper invokeHelper = null;
	JDialog progressDialog = null;
	JFrame frame = null;
	String developerKitHome = "";

	public DisplayDataCommand(JFrame frm, String devkitHome)
	{
		try
		{
			caption = "Display Tracking Data";
			category = "Java Samples";
			message = "Display Tracking Data";
			name = "DisplayCommand_DisplayCommand";
			toolTip = "Display Tracking Data.";
			helpFile = "";
			this.frame = frm;
			this.developerKitHome = devkitHome;
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}

	/**
	 * The Command's onCreate method, invoked when command is created
	 */
	public void onCreate(Object hook)
	{
		try
		{
			hookHelper = new HookHelper();
			hookHelper.setHookByRef(hook);

			IActiveView activeView = hookHelper.getActiveView();

			if (activeView == null)
			{
				hookHelper = null;
				enabled = false;
			}
			else
			{
				enabled = true;
			}
		}
		catch (IOException ex)
		{
			ex.printStackTrace();
		}
	}

	/**
	 * The Commands onClick() method, invoked when user clicks on the command button
	 */
	public void onClick()
	{
		try
		{
			if (!isConnected)
			{
				getProgressDialog().setVisible(true);

				// get the map container
				Object mapObj = null;
				if (hookHelper != null)
				{
					if (hookHelper.getHook() instanceof IToolbarControl2)
					{
						IToolbarControl2 toolbarControl = (IToolbarControl2) hookHelper.getHook();
						mapObj = toolbarControl.getBuddy();
					}
					else
					{
						mapObj = hookHelper.getHook();
					}
				}

				// load the Tracking Analyst extension
				ITrackingEnvironment trackingEnv = setupTrackingEnv(mapObj);

				// get the default temporal reference
				TemporalReference temporalReference = (TemporalReference) trackingEnv.getDefaultTemporalReference();
				// set the mode to historic, since you need to do playback
				temporalReference.setTemporalMode(enumTemporalMode.enumHistoric);
				// set the units of the temporal period to days
				temporalReference.setTemporalPeriodUnits(enumTemporalUnits.enumDays);
				// set the update mode to manual so that it will be controled by
				// the application
				trackingEnv.getDisplayManager().setManualUpdate(true);
				// set the temporal perspective to Aug 03 2000 7PM.
				temporalReference.setTemporalPerspective("8/3/2000 7:0:00 PM");

				// create a temporal operator that will serve as a base time for
				// the tracking envi
				TemporalOperator temporalOpBaseTime = new TemporalOperator();

				// set the base time to 6PM, Aug 3 2000
				short month = 8;
				short dayOfMonth = 3;
				short hour = 18;
				short minute = 0;
				short second = 0;
				short milliSecond = 0;
				temporalOpBaseTime.setDateTime(2000, month, dayOfMonth, hour, minute, second, milliSecond);

				// open the shapefile with the recorded data
				IFeatureClass featureClass = getShapefile();
				if (featureClass == null)
				{
					return;
				}

				String labelField = "EVENTID";

				// create the renderer for the temporal layer
				CoTrackSymbologyRenderer temporalRenderer = setRenderer(featureClass, "TA_DATE", "EVENTID");
				temporalRenderer.setTemporalFieldName("TA_Date");
				temporalRenderer.setTemporalObjectColumnName("Eventid");
				temporalRenderer.setLabelFieldName(labelField);
				temporalRenderer.setLabelFeatures(true);

				// create the temporal layer for the playback data
				temporalLayer = new TemporalFeatureLayer();
				// assign the featureclass for the layer
				temporalLayer.setFeatureClassByRef(featureClass);
				// set the base time to initialize the time window of the layer
				temporalLayer.setRelativeTimeOperator(temporalOpBaseTime);
				// set the renderer for the temporal layer
				temporalLayer.setRendererByRef((IFeatureRenderer) temporalRenderer);
				// set the flag in order to display the track of previous
				// locations
				temporalLayer.setDisplayOnlyLastKnownEvent(false);
				// initialize labels for the envent name
				temporalLayer.setLabelFieldName(labelField);
				temporalLayer.setLabelFeatures(true);
				TextSymbol textSymbol = setupLayerLabels();
				temporalLayer.setTextSymbolByRef(textSymbol);
				temporalRenderer.setTextSymbolByRef(textSymbol);

				/*
				 * Create the InvokeHelper object which is a timer needed to increment the time stamp. InvokeHelper is
				 * derived from the Swing Timer class, which uses just one thread for all timers. It deals with
				 * scheduling actions and putting its thread to sleep internally in a way that scales to large numbers
				 * of timers. The other important feature of this timer class is that the Action's actionPerformed
				 * method runs on the event dispatching thread. As a result, you don't have to bother with an explicit
				 * invokeLater call.
				 */
				invokeHelper = new InvokeHelper(1000);// 1000 milliseconds delay between firing of each event
				invokeHelper.setActiveView(hookHelper.getActiveView());
				invokeHelper.setTrackingEnv(trackingEnv);
				invokeHelper.setBaseTime(temporalOpBaseTime);
				invokeHelper.setTemporalLayer(temporalLayer);

				// enable the timer
				invokeHelper.start();

				// add the temporal layer to the map
				hookHelper.getFocusMap().addLayer(temporalLayer);

				getProgressDialog().setVisible(false);
			}
			else
			{
				// disable the timer
				invokeHelper.stop();

				if (temporalLayer == null)
				{
					return;
				}

				// remode the layer
				hookHelper.getFocusMap().deleteLayer(temporalLayer);
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		isConnected = !isConnected;
	}

	/**
	 * Finds and returns specifed FC in specified path
	 */
	public IFeatureClass getShapefile() throws Exception
	{
		String path = this.developerKitHome + File.separator + "java" + File.separator + "samples" + File.separator + "data"
				+ File.separator + "tracking";
		String fcName = "2000_hrcn";

		// determine type of GDB
		IWorkspaceFactory wsFactory = null;
		if (!path.endsWith(".gdb") && !path.endsWith(".mdb") && new File(path).isDirectory())
		{
			wsFactory = new ShapefileWorkspaceFactory();
		}
		else
		{
			throw new Exception("Supplied path contains FGDB, PGDB or is not a folder.");
		}

		// create workspace
		Workspace ws = new Workspace(wsFactory.openFromFile(path, 0));
		if (ws == null)
		{
			throw new Exception("Workspace is null in getShapefile().");
		}

		// check if shapefile exists in path
		System.out.println("Searching for " + fcName + ".shp in " + path + "...");
		IEnumDataset enumFCDS = ws.getDatasets(esriDatasetType.esriDTFeatureClass);
		IDataset fcDS = enumFCDS.next();
		if (fcDS != null)
		{
			while (fcDS != null)
			{
				if (fcDS.getName().equalsIgnoreCase(fcName))
				{
					System.out.println(fcName + ".shp found.");
					return new IFeatureClassProxy(fcDS);
				}

				fcDS = enumFCDS.next();
			}
		}

		System.err.println(fcName + ".shp not found in " + path + ".");
		return null;
	}

	/**
	 * Sets up the Tracking Environment
	 * 
	 * @param mapObj
	 * @return
	 */
	private ITrackingEnvironment setupTrackingEnv(Object mapObj)
	{
		TrackingEnvironment trackingEnv;
		try
		{
			ExtensionManager extentionManager = new ExtensionManager();

			UID uid = new UID();
			uid.setValue("esriTrackingAnalyst.TrackingEngineUtil");

			((IExtensionManagerAdmin) extentionManager).addExtension(uid, mapObj);

			trackingEnv = new TrackingEnvironment();
			trackingEnv.initialize(mapObj);
			trackingEnv.setEnableTemporalDisplayManagement(true);

			return trackingEnv;
		}
		catch (UnknownHostException e)
		{
			e.printStackTrace();
		}
		catch (AutomationException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Sets Renderers for the the shapefile using temporal field (date) and the event id
	 * 
	 * @param featureClass
	 * @param temporalField
	 * @param eventName
	 * @return
	 */
	private CoTrackSymbologyRenderer setRenderer(IFeatureClass featureClass, String temporalField, String eventName)
	{
		try
		{
			CoTrackSymbologyRenderer trackingRenderer = new CoTrackSymbologyRenderer();
			trackingRenderer.setTemporalFieldName(temporalField);
			trackingRenderer.setTemporalObjectColumnName(eventName);
			trackingRenderer.setTimeSymbologyMethod(enumTemporalSymbolizationMethod.enumColor);

			// this is a desktop only code which requires assemblies CartoUI and
			// Framework
			// IRendererPropertyPage rendererPropPage = new
			// UniqueValuePropertyPageClass();

			// enable the most current renderer
			IUniqueValueRenderer uniqueValrenderer = createUniqueValueRenderer(featureClass, eventName);
			if (null != uniqueValrenderer)
			{
				trackingRenderer.setMostCurrentAdvRendererByRef((IFeatureRenderer) uniqueValrenderer);
				trackingRenderer.setMostCurrentAdvRendererEnabled(true);
			}

			// set the track renderer
			uniqueValrenderer = createTrackUniqueValueRenderer(featureClass, eventName);
			if (uniqueValrenderer != null)
			{
				trackingRenderer.setTrackSymbologyRendererByRef((IFeatureRenderer) uniqueValrenderer);
				trackingRenderer.setTrackRendererEnabled(true);
				trackingRenderer.setSmoothTracks(true);

				// this is a desktop only code which requires assemblies CartoUI
				// and Framework
				// ((ITemporalRenderer2)temporalRenderer).PropPageTrackRenderer
				// = rendererPropPage.ClassID;
			}

			return trackingRenderer;
		}
		catch (UnknownHostException e)
		{
			e.printStackTrace();
		}
		catch (AutomationException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Creates a Unique Value Renderer for the feature class
	 * 
	 * @param featureClass
	 * @param fieldName
	 * @return
	 */
	private IUniqueValueRenderer createUniqueValueRenderer(IFeatureClass featureClass, String fieldName)
	{
		try
		{
			RgbColor color = new RgbColor();
			color.setRed(255);
			color.setBlue(0);
			color.setGreen(0);

			CharacterMarkerSymbol charMarkersymbol = new CharacterMarkerSymbol();

			StdFont font = new StdFont();
			font.setName("ESRI Default Marker");
			font.setBold(false);
			font.setItalic(false);
			font.setSize(12);

			charMarkersymbol.setFont(font);
			charMarkersymbol.setCharacterIndex(96);
			charMarkersymbol.setSize(12.0);
			charMarkersymbol.setColor(color);

			RandomColorRamp randomColorRamp = new RandomColorRamp();
			randomColorRamp.setMinSaturation(20);
			randomColorRamp.setMaxSaturation(40);
			randomColorRamp.setMaxValue(85);
			randomColorRamp.setMaxValue(100);
			randomColorRamp.setStartHue(75);
			randomColorRamp.setEndHue(190);
			randomColorRamp.setUseSeed(true);
			randomColorRamp.setSeed(45);

			UniqueValueRenderer uniqueRenderer = new UniqueValueRenderer();
			uniqueRenderer.setFieldCount(1);
			uniqueRenderer.setField(0, fieldName);
			uniqueRenderer.setDefaultSymbol(charMarkersymbol);
			uniqueRenderer.setUseDefaultSymbol(true);

			Random rand = new Random(255);
			boolean bValFound = false;
			IFeatureCursor featureCursor = featureClass.search(null, true);
			IFeature feature = null;
			String val = "";

			int fieldID = featureClass.findField(fieldName);
			if (fieldID == -1)
			{
				return uniqueRenderer;
			}

			while ((feature = featureCursor.nextFeature()) != null)
			{
				bValFound = false;
				val = feature.getValue(fieldID).toString();
				for (int i = 0; i < uniqueRenderer.getValueCount(); i++)
				{
					if (uniqueRenderer.getValue(i).equals(val))
					{
						bValFound = true;
					}
				}

				if (!bValFound)// need to add the value to the renderer
				{
					color.setRed(rand.nextInt(255));
					color.setBlue(rand.nextInt(255));
					color.setGreen(rand.nextInt(255));

					StdFont font1 = new StdFont();
					font1.setName("ESRI Default Marker");
					font1.setBold(false);
					font1.setItalic(false);
					font1.setSize(10);

					charMarkersymbol = new CharacterMarkerSymbol();
					charMarkersymbol.setFont(font1);
					rand.setSeed(40);
					charMarkersymbol.setCharacterIndex(rand.nextInt(118));
					charMarkersymbol.setSize(20);
					charMarkersymbol.setColor(color);

					// add the value to the renderer
					uniqueRenderer.addValue(val, "name", charMarkersymbol);
				}
			}

			return uniqueRenderer;
		}
		catch (UnknownHostException e)
		{
			e.printStackTrace();
		}
		catch (AutomationException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Creates a Track Unique Value Renderer for Track Symbology for the shape file
	 * 
	 * @param featureClass
	 * @param fieldName
	 * @return
	 */
	private IUniqueValueRenderer createTrackUniqueValueRenderer(IFeatureClass featureClass, String fieldName)
	{
		try
		{
			RgbColor color = new RgbColor();
			color.setRed(0);
			color.setBlue(0);
			color.setGreen(255);

			SimpleLineSymbol simpleLineSymbol = new SimpleLineSymbol();
			simpleLineSymbol.setColor(color);
			simpleLineSymbol.setStyle(esriSimpleLineStyle.esriSLSSolid);
			simpleLineSymbol.setWidth(1.0);

			UniqueValueRenderer uniqueRenderer = new UniqueValueRenderer();
			uniqueRenderer.setFieldCount(1);
			uniqueRenderer.setField(0, fieldName);
			uniqueRenderer.setDefaultSymbol(simpleLineSymbol);
			uniqueRenderer.setUseDefaultSymbol(true);

			Random rand = new Random(255);
			boolean bValFound = false;
			IFeatureCursor featureCursor = featureClass.search(null, true);
			IFeature feature = null;
			String val = "";
			int fieldID = featureClass.findField(fieldName);
			if (fieldID == -1)
			{
				return uniqueRenderer;
			}

			while ((feature = featureCursor.nextFeature()) != null)
			{
				bValFound = false;
				val = feature.getValue(fieldID).toString();
				for (int i = 0; i < uniqueRenderer.getValueCount(); i++)
				{
					if (uniqueRenderer.getValue(i).equals(val))
					{
						bValFound = true;
					}
				}

				if (!bValFound)// need to add the value to the renderer
				{
					color.setRed(rand.nextInt(255));
					color.setBlue(rand.nextInt(255));
					color.setGreen(rand.nextInt(255));

					simpleLineSymbol = new SimpleLineSymbol();
					simpleLineSymbol.setColor(color);
					simpleLineSymbol.setStyle(esriSimpleLineStyle.esriSLSSolid);
					simpleLineSymbol.setWidth(1.0);

					// add the value to the renderer
					uniqueRenderer.addValue(val, "name", simpleLineSymbol);
				}
			}

			return uniqueRenderer;
		}
		catch (UnknownHostException e)
		{
			e.printStackTrace();
		}
		catch (AutomationException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}

		return null;
	}

	/**
	 * Sets up labels for tacking layer
	 * 
	 * @param trackingLayer
	 * @param labelField
	 */
	private TextSymbol setupLayerLabels()
	{
		try
		{

			// create text symbol
			TextSymbol textSymbol = new TextSymbol();

			RgbColor color = new RgbColor();
			color.setRed(255);
			color.setBlue(0);
			color.setGreen(0);

			StdFont font = new StdFont();
			font.setName("Arial");
			font.setBold(true);
			font.setItalic(false);
			// font.setSize(15);

			textSymbol.setColor(color);
			textSymbol.setSize(15);
			textSymbol.setFont(font);
			textSymbol.setHorizontalAlignment(esriTextHorizontalAlignment.esriTHARight);
			textSymbol.setVerticalAlignment(esriTextVerticalAlignment.esriTVABaseline);

			return textSymbol;
		}
		catch (AutomationException e)
		{
			e.printStackTrace();
		}
		catch (UnknownHostException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		return null;
	}

	/*
	 * Creates a Dialog box with a Progress bar in it. @return
	 */
	private JDialog getProgressDialog()
	{
		if (progressDialog == null)
		{
			progressDialog = new JDialog();
			progressDialog.setTitle("Loading Tracking data...");
			JProgressBar bar = new JProgressBar();
			bar.setBorderPainted(true);
			bar.setIndeterminate(true);
			progressDialog.setLocationRelativeTo(frame);
			progressDialog.setLocation(frame.getWidth() / 2, frame.getHeight() / 2);
			progressDialog.add(bar, BorderLayout.PAGE_START);
			progressDialog.pack();
			progressDialog.setSize(250, 50);
		}

		return progressDialog;
	}

	public boolean isEnabled()
	{
		return true;
	}

	public boolean isChecked()
	{
		return checked;
	}

	public String getName()
	{
		return name;
	}

	public String getCaption()
	{
		return caption;
	}

	public String getTooltip()
	{
		return toolTip;
	}

	public String getMessage()
	{
		return message;
	}

	public String getHelpFile()
	{
		return helpFile;
	}

	public int getHelpContextID()
	{
		return 0;
	}

	public String getCategory()
	{
		return category;
	}
}