/* Copyright 2010 ESRI
* 
* All rights reserved under the copyright laws of the United States
* and applicable international laws, treaties, and conventions.
* 
* You may freely redistribute and use this sample code, with or
* without modification, provided you include the original copyright
* notice and use restrictions.
* 
* See the use restrictions at http://help.arcgis.com/en/sdk/10.0/usageRestrictions.htm
* 
*/
package arcgissamples.trackinganalyst.command;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.UnknownHostException;

import javax.swing.Timer;

import com.esri.arcgis.carto.IActiveView;
import com.esri.arcgis.carto.esriViewDrawPhase;
import com.esri.arcgis.interop.AutomationException;
import com.esri.arcgis.trackinganalyst.ITemporalLayer;
import com.esri.arcgis.trackinganalyst.ITemporalOperator3;
import com.esri.arcgis.trackinganalyst.ITrackingEnvironment;
import com.esri.arcgis.trackinganalyst.TemporalOperator;
import com.esri.arcgis.trackinganalyst.enumTemporalOperatorUnits;

public class InvokeHelper extends Timer
{
	private IActiveView activeView = null;
	private ITemporalLayer temporalLayer = null;
	private TemporalOperator tempOpIncrement = null;// ITemporalOperator3 tempOpIncrement = null;
	private TemporalOperator baseTime = null;// ITemporalOperator3 baseTime = null;
	private ITrackingEnvironment trackingEnv = null;
	boolean isFirst = true;

	public InvokeHelper()
	{
		this(1000);
	}

	public InvokeHelper(int delay)
	{
		super(delay, null);

		ActionListener taskPerformer = new ActionListener()
		{
			public void actionPerformed(ActionEvent evt)
			{
				incrementTimeStamp();
			}
		};

		addActionListener(taskPerformer);
	}

	/**
	 * this method gets executed on the main thread and it increments the temporal environment time stamp
	 */
	private void incrementTimeStamp()
	{
		try
		{
			if (isFirst)
			{
				// create the temporal increment object
				tempOpIncrement = new TemporalOperator();
				tempOpIncrement.setInterval(6.0, enumTemporalOperatorUnits.enumTemporalOperatorHours);

				isFirst = false;
			}

			if (baseTime == null)
			{
				return;
			}

			// increment the base time to match the 'current' time
			baseTime.add(tempOpIncrement);// (ITemporalOperator)

			String date = baseTime.getAsString("%c");
			System.out.println(date);

			// increment the timestamp
			trackingEnv.getDefaultTemporalReference().setTemporalPerspective(baseTime.getAsDate()); // date);

			// refresh the display
			activeView.partialRefresh(esriViewDrawPhase.esriViewGeography, temporalLayer, null);
		}
		catch (UnknownHostException e)
		{
			e.printStackTrace();
		}
		catch (AutomationException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		catch (java.lang.RuntimeException re)
		{
			// do nothing
		}
	}

	// activeView
	public IActiveView getActiveView()
	{
		return this.activeView;
	}

	public void setActiveView(IActiveView activeView)
	{
		this.activeView = activeView;
	}

	// trackingEnvironment
	public ITrackingEnvironment getTrackingEnv()
	{
		return trackingEnv;
	}

	public void setTrackingEnv(ITrackingEnvironment trackingEnv)
	{
		this.trackingEnv = trackingEnv;
	}

	// BaseTime
	public ITemporalOperator3 getBaseTime()
	{
		return this.baseTime;
	}

	public void setBaseTime(TemporalOperator baseTime)
	{
		this.baseTime = baseTime;
	}

	// temporalLayer
	public ITemporalLayer getTemporalLayer()
	{
		return this.temporalLayer;
	}

	public void setTemporalLayer(ITemporalLayer temporalLayer)
	{
		this.temporalLayer = temporalLayer;
	}
}
